# Update IP catalog
update_ip_catalog -rebuild

# Disable display port
startgroup
set_property CONFIG.PSU__DISPLAYPORT__PERIPHERAL__ENABLE {0} [get_bd_cells zynq_ultra_ps_e_0]
endgroup

# Enable HP0
startgroup
set_property CONFIG.PSU__USE__S_AXI_GP2 {1} [get_bd_cells zynq_ultra_ps_e_0]
endgroup

# Enable interrupt input
startgroup
set_property CONFIG.PSU__USE__IRQ0 {1} [get_bd_cells zynq_ultra_ps_e_0]
endgroup

# Add clock wizard
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:clk_wiz:6.0 vga_clk_gen
endgroup

# Configure clock wizard to generate 74.25 MHz pixel clock
set_property -dict [list \
  CONFIG.CLKOUT1_JITTER {250.227} \
  CONFIG.CLKOUT1_PHASE_ERROR {394.762} \
  CONFIG.CLKOUT1_REQUESTED_OUT_FREQ {74.25} \
  CONFIG.CLK_OUT1_PORT {clk_74_25_MHz} \
  CONFIG.MMCM_CLKFBOUT_MULT_F {111.375} \
  CONFIG.MMCM_CLKOUT0_DIVIDE_F {15.000} \
  CONFIG.MMCM_DIVCLK_DIVIDE {10} \
  CONFIG.RESET_PORT {resetn} \
  CONFIG.RESET_TYPE {ACTIVE_LOW} \
] [get_bd_cells vga_clk_gen]

# Add VDMA
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:axi_vdma:6.3 axi_vdma_0
endgroup

# Configure VDMA
set_property -dict [list \
  CONFIG.c_include_s2mm {0} \
  CONFIG.c_m_axi_mm2s_data_width {128} \
  CONFIG.c_mm2s_linebuffer_depth {1024} \
  CONFIG.c_mm2s_max_burst_length {8} \
  CONFIG.c_num_fstores {1} \
] [get_bd_cells axi_vdma_0]


# Remove vga_ctrl_0 core
delete_bd_objs [get_bd_nets zynq_ultra_ps_e_0_pl_clk1] [get_bd_nets vga_ctrl_0_h_sync] [get_bd_nets vga_ctrl_0_v_sync] [get_bd_nets vga_ctrl_0_red_out] [get_bd_nets vga_ctrl_0_green_out] [get_bd_nets vga_ctrl_0_blue_out] [get_bd_cells vga_ctrl_0]

#*********************************************************
# Add VGA OUT hierarchy
create_bd_cell -type hier vga_out

  # Add subset converter
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:axis_subset_converter:1.1 vga_out/axis_subset_converter_0
  endgroup

  # ABGR to RBG
  set_property -dict [list CONFIG.M_TDATA_NUM_BYTES.VALUE_SRC USER CONFIG.S_TDATA_NUM_BYTES.VALUE_SRC USER] [get_bd_cells vga_out/axis_subset_converter_0]
  set_property -dict [list \
    CONFIG.M_TDATA_NUM_BYTES {3} \
    CONFIG.S_TDATA_NUM_BYTES {4} \
    CONFIG.TDATA_REMAP {tdata[7:0],tdata[23:16],tdata[15:8]} \
  ] [get_bd_cells vga_out/axis_subset_converter_0]

  # Add video timing controller
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:v_tc:6.2 vga_out/v_tc_0
  endgroup

  # Disable AXI Lite and detector
  set_property -dict [list \
    CONFIG.HAS_AXI4_LITE {false} \
    CONFIG.enable_detection {false} \
  ] [get_bd_cells vga_out/v_tc_0]

  # Set fixed timing to 1280x720p60
  set_property CONFIG.VIDEO_MODE {720p} [get_bd_cells vga_out/v_tc_0]

  # Add video out core
  startgroup
  create_bd_cell -type ip -vlnv xilinx.com:ip:v_axi4s_vid_out:4.0 vga_out/v_axi4s_vid_out_0
  endgroup

  # Set video out as a master
  set_property -dict [list \
    CONFIG.C_HAS_ASYNC_CLK {1} \
    CONFIG.C_VTG_MASTER_SLAVE {1} \
  ] [get_bd_cells vga_out/v_axi4s_vid_out_0]

  # Add VGA out core
  startgroup
  create_bd_cell -type ip -vlnv UTIA:utia:vga_out:1.0 vga_out/vga_out_0
  endgroup

  # Configure VGA out
  set_property -dict [list \
    CONFIG.C_COLOR_IN_WIDTH {8} \
    CONFIG.C_COLOR_OUT_WIDTH {4} \
  ] [get_bd_cells vga_out/vga_out_0]

  # Create ports
  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:axis_rtl:1.0 vga_out/VID_IN_AXIS
  create_bd_pin -dir I vga_out/aclk
  create_bd_pin -dir I vga_out/aresetn
  create_bd_pin -dir I vga_out/vid_out_clk
  create_bd_pin -dir O -from 3 -to 0 vga_out/R
  create_bd_pin -dir O -from 3 -to 0 vga_out/G
  create_bd_pin -dir O -from 3 -to 0 vga_out/B
  create_bd_pin -dir O vga_out/VS
  create_bd_pin -dir O vga_out/HS

  # Make connections
  connect_bd_intf_net [get_bd_intf_pins vga_out/VID_IN_AXIS] [get_bd_intf_pins vga_out/axis_subset_converter_0/S_AXIS]

  connect_bd_net [get_bd_pins vga_out/aclk] [get_bd_pins vga_out/axis_subset_converter_0/aclk]
  connect_bd_net [get_bd_pins vga_out/aresetn] [get_bd_pins vga_out/axis_subset_converter_0/aresetn]
  connect_bd_net [get_bd_pins vga_out/vid_out_clk] [get_bd_pins vga_out/v_tc_0/clk]

  connect_bd_net [get_bd_pins vga_out/R] [get_bd_pins vga_out/vga_out_0/r_out]
  connect_bd_net [get_bd_pins vga_out/G] [get_bd_pins vga_out/vga_out_0/g_out]
  connect_bd_net [get_bd_pins vga_out/B] [get_bd_pins vga_out/vga_out_0/b_out]
  connect_bd_net [get_bd_pins vga_out/VS] [get_bd_pins vga_out/vga_out_0/vs_out]
  connect_bd_net [get_bd_pins vga_out/HS] [get_bd_pins vga_out/vga_out_0/hs_out]

  connect_bd_intf_net [get_bd_intf_pins vga_out/v_axi4s_vid_out_0/video_in] [get_bd_intf_pins vga_out/axis_subset_converter_0/M_AXIS]
  connect_bd_intf_net [get_bd_intf_pins vga_out/v_tc_0/vtiming_out] [get_bd_intf_pins vga_out/v_axi4s_vid_out_0/vtiming_in]

  connect_bd_net [get_bd_pins vga_out/aclk] [get_bd_pins vga_out/v_axi4s_vid_out_0/aclk]
  connect_bd_net [get_bd_pins vga_out/aresetn] [get_bd_pins vga_out/v_axi4s_vid_out_0/aresetn]

  connect_bd_net [get_bd_pins vga_out/vid_out_clk] [get_bd_pins vga_out/v_axi4s_vid_out_0/vid_io_out_clk]
  connect_bd_net [get_bd_pins vga_out/vid_out_clk] [get_bd_pins vga_out/vga_out_0/clk]

  connect_bd_net [get_bd_pins vga_out/vga_out_0/vid_in] [get_bd_pins vga_out/v_axi4s_vid_out_0/vid_data]
  connect_bd_net [get_bd_pins vga_out/vga_out_0/de_in] [get_bd_pins vga_out/v_axi4s_vid_out_0/vid_active_video]
  connect_bd_net [get_bd_pins vga_out/vga_out_0/vs_in] [get_bd_pins vga_out/v_axi4s_vid_out_0/vid_vsync]
  connect_bd_net [get_bd_pins vga_out/vga_out_0/hs_in] [get_bd_pins vga_out/v_axi4s_vid_out_0/vid_hsync]

#*********************************************************
# Configure AXI interconnect
set_property CONFIG.NUM_MI {2} [get_bd_cells axi_interconnect_0]

# Fix AXI interconnect main reset
disconnect_bd_net /S01_ARESETN_1 [get_bd_pins axi_interconnect_0/ARESETN]
connect_bd_net [get_bd_pins axi_interconnect_0/ARESETN] [get_bd_pins proc_sys_reset_0/interconnect_aresetn]

# Add concat for interrupts
startgroup
create_bd_cell -type ip -vlnv xilinx.com:ip:xlconcat:2.1 IRQs
endgroup
set_property CONFIG.NUM_PORTS {1} [get_bd_cells IRQs]

#*********************************************************
# Make connections

connect_bd_net [get_bd_pins zynq_ultra_ps_e_0/saxihp0_fpd_aclk] [get_bd_pins zynq_ultra_ps_e_0/pl_clk0]

connect_bd_net [get_bd_pins vga_clk_gen/resetn] [get_bd_pins zynq_ultra_ps_e_0/pl_resetn0]
connect_bd_net [get_bd_pins vga_clk_gen/clk_in1] [get_bd_pins zynq_ultra_ps_e_0/pl_clk0]
connect_bd_net [get_bd_pins vga_clk_gen/locked] [get_bd_pins proc_sys_reset_0/dcm_locked]
connect_bd_net [get_bd_pins vga_clk_gen/clk_74_25_MHz] [get_bd_pins vga_out/vid_out_clk]

connect_bd_intf_net [get_bd_intf_pins axi_vdma_0/S_AXI_LITE] -boundary_type upper [get_bd_intf_pins axi_interconnect_0/M01_AXI]
connect_bd_intf_net [get_bd_intf_pins axi_vdma_0/M_AXI_MM2S] [get_bd_intf_pins zynq_ultra_ps_e_0/S_AXI_HP0_FPD]
connect_bd_intf_net [get_bd_intf_pins axi_vdma_0/M_AXIS_MM2S] -boundary_type upper [get_bd_intf_pins vga_out/VID_IN_AXIS]
connect_bd_net [get_bd_pins axi_vdma_0/s_axi_lite_aclk] [get_bd_pins zynq_ultra_ps_e_0/pl_clk0]
connect_bd_net [get_bd_pins axi_vdma_0/m_axi_mm2s_aclk] [get_bd_pins zynq_ultra_ps_e_0/pl_clk0]
connect_bd_net [get_bd_pins axi_vdma_0/m_axis_mm2s_aclk] [get_bd_pins zynq_ultra_ps_e_0/pl_clk0]
connect_bd_net [get_bd_pins axi_interconnect_0/M01_ACLK] [get_bd_pins zynq_ultra_ps_e_0/pl_clk0]
connect_bd_net [get_bd_pins axi_vdma_0/axi_resetn] [get_bd_pins proc_sys_reset_0/peripheral_aresetn]
connect_bd_net [get_bd_pins axi_interconnect_0/M01_ARESETN] [get_bd_pins proc_sys_reset_0/peripheral_aresetn]

connect_bd_net [get_bd_pins IRQs/In0] [get_bd_pins axi_vdma_0/mm2s_introut]
connect_bd_net [get_bd_pins IRQs/dout] [get_bd_pins zynq_ultra_ps_e_0/pl_ps_irq0]

connect_bd_net [get_bd_pins vga_out/aclk] [get_bd_pins zynq_ultra_ps_e_0/pl_clk0]
connect_bd_net [get_bd_pins vga_out/aresetn] [get_bd_pins proc_sys_reset_0/peripheral_aresetn]

connect_bd_net [get_bd_ports VGA_R] [get_bd_pins vga_out/R]
connect_bd_net [get_bd_ports VGA_G] [get_bd_pins vga_out/G]
connect_bd_net [get_bd_ports VGA_B] [get_bd_pins vga_out/B]
connect_bd_net [get_bd_ports VGA_VS] [get_bd_pins vga_out/VS]
connect_bd_net [get_bd_ports VGA_HS] [get_bd_pins vga_out/HS]

regenerate_bd_layout -routing

# Assign addresses to unmapped peripherals
assign_bd_address -offset 0x80010000 -range 0x00010000 -target_address_space /zynq_ultra_ps_e_0/Data [get_bd_addr_segs axi_vdma_0/S_AXI_LITE/Reg] -force


#save
save_bd_design

#save project XPR name
global proj_xpr
set proj_xpr [current_project]
append proj_xpr .xpr

#close project
close_project

# reopen project
open_project $proj_xpr

# open block design
open_bd_design [current_project].srcs/sources_1/bd/zusys/zusys.bd

#validate
#validate_bd_design




